/* ============================================================
 * File  : guilelements.h
 * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
 * Date  : 2004-02-12
 * Description : streamtuned gui
 *
 * Copyright 2003 by Eric Giesselbach

 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */


#ifndef GUIELEMENTS_H
#define GUIELEMENTS_H

#include <qwidget.h>
#include <qlabel.h>
#include <qrect.h>
#include <qtimer.h>
#include <qintdict.h>
#include <qpointarray.h>
#include <qbitmap.h>

#include "../libs/streambrowser.h"  // Spectrum


class GuiParameters
{
    
  public:
   
    GuiParameters()
    {
      gaugeColor  = QColor(0x00, 0x00, 0xff);
      textLoColor = QColor(0xaf, 0xaf, 0x01);
      textSelectedColor = QColor(0x00, 0xFF, 0x00);
      textHighlightColor = Qt::green;
      playerBoxEdge = Qt::red;
      playerBoxFont = Qt::red;
      harvesterBoxEdge = Qt::red;
      harvesterBoxFont = Qt::red;
      sampleTextWidth = 125;
    };
    
    QColor gaugeColor;
    QColor textLoColor;
    QColor textSelectedColor;
    QColor textHighlightColor;
    QColor playerBoxEdge;
    QColor playerBoxFont;
    QColor harvesterBoxEdge;
    QColor harvesterBoxFont;
    uint   sampleTextWidth;
};


class Gauge : public QWidget
{
   Q_OBJECT

   public:
     Gauge(QWidget *parent, GuiParameters* params, unsigned int x, unsigned int y, unsigned int w, unsigned int h);

     void setPercentage(int percent);
     void paintEvent( QPaintEvent * );

   private:
     int percentage;
     int barWidth;
     int gaugeWidth;
     float div;
     GuiParameters *guiParameters;

};

class FFTBox : public QWidget
{
   Q_OBJECT

   public:
     FFTBox(QWidget *parent, GuiParameters* params, int barcnt, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
     ~FFTBox();

     void paintEvent( QPaintEvent * );

   public slots:
     void setPercentage(Spectrum* percent);

   private:
     Spectrum *percentage;
     int barCount;
     int barWidth;
     int gaugeWidth;
     float div;
     int textPosX, textPosY;
     QPixmap fftPict;
     QPixmap bckPict;
     QPixmap drawPict;
     QBitmap fftMask;
     bool go;
     GuiParameters *guiParameters;
};

class Led : QWidget
{
  Q_OBJECT

  public:
    Led(QWidget *parent, unsigned int x, unsigned int y, bool tilt);
    void paintEvent( QPaintEvent * );
    void setLightness(int light);  // turns off blinking
    void setBlinking(bool blink);
    void setColor(QColor col);
    void setIndicator(bool on);

  private slots:
    void toggleBlink();

  private:
    int lightness;
    bool tilted;
    QColor color;
    QPointArray points;
    QRect circle;
    int h, s, v; // hue, sat, value

    QTimer *blinkTimer;
    int blinkHi, blinkLo;
    bool blinkIsHi;
    bool indicator;
};

class PlayerMessageBox : QWidget
{
  Q_OBJECT

  public:
    PlayerMessageBox(QWidget *parent, GuiParameters* params, unsigned int x, unsigned int y, unsigned int w);

    void addMessage(QString message);

  public slots:
    void clearMessages();

  protected:
    void paintEvent( QPaintEvent * );

  private:
    QTimer *hideTimer;
    QPointArray box, lab;
    QLabel *line1, *line2, *line3;
    QLabel* PlayerMessageBox::prepLabel(QLabel *label, unsigned int index, unsigned int w);
    GuiParameters *guiParameters;
};

class HarvesterMessageBox : QWidget
{
  Q_OBJECT

  public:
    HarvesterMessageBox(QWidget *parent, GuiParameters* params, unsigned int x, unsigned int y, unsigned int w);

    void setMessage(QString message);

  protected:
    void paintEvent( QPaintEvent * );

  private:
    QPointArray box, lab;
    QLabel *line;
    QLabel* prepLabel(QLabel *label, unsigned int w);
    GuiParameters *guiParameters;
};

class RecLed : QWidget
{
  Q_OBJECT

  public:
    RecLed(QWidget *parent, unsigned int x, unsigned int y);
    void paintEvent( QPaintEvent * );
    void setLightness(int light);  // turns off blinking
    void setColor(QColor col);
    void setIndicator(bool on);

  private:
    int lightness;
    QColor color;
    QRect circle;
    int h, s, v; // hue, sat, value

    bool indicator;
};

class Logo : public QLabel
{
  Q_OBJECT

  public:
    Logo(QWidget *parent, int x, int y, int w, int h);
    void parentHasFocus( bool hasFocus);
};

class MessageLabel : public QLabel
{
  Q_OBJECT

  public:
    MessageLabel(QWidget *parent, GuiParameters* params, int x, int y, int w, int h);
  private:
    QFont normFont;
    QFont boldFont;
    GuiParameters *guiParameters;
};

class ItemLabel : public QLabel
{
  Q_OBJECT

  public:
    ItemLabel(QWidget *parent, GuiParameters* params, int x, int y, int w, int h);
    void setSelected(bool cursor);
    void setHighLight(bool cursor);
    void setColor(QColor& color, bool bold);
  private:
    bool selected;
    QFont normFont;
    QFont boldFont;
    GuiParameters *guiParameters;
};

class ItemIcon : public QLabel
{
  Q_OBJECT

  public:
    ItemIcon(QWidget *parent, GuiParameters* params, int x, int y, int w, int h);
    void setSelected(bool cursor);
    void setHighLight(bool cursor);
  private:
    bool selected;
    GuiParameters *guiParameters;
};

class FolderLabel : public QLabel
{
  Q_OBJECT

  public:
    FolderLabel(QWidget *parent, GuiParameters* params, int x, int y, int w, int h);
    void setSelected(bool cursor);
    void setHighLight(bool cursor);
  private:
    bool selected;
    QFont normFont;
    QFont boldFont;
    GuiParameters *guiParameters;
};

    
#endif /* GUIELEMENTS_H */
